diff --git a/libavformat/applehttp.c b/libavformat/applehttp.c
index 6c4d56d3056489b9293b61a9c1752e7486cff4c8..9ab5927b922cf7a167f032cc1b0b633e0b1059ed 100644
--- a/libavformat/applehttp.c
+++ b/libavformat/applehttp.c
@@ -377,10 +377,15 @@ static int read_data(void *opaque, uint8_t *buf, int buf_size)
 restart:
     if (!v->input) {
 reload:
         /* If this is a live stream and target_duration has elapsed since
          * the last playlist reload, reload the variant playlists now. */
+        int64_t reload_interval = v->n_segments > 0 ?
+                                  v->segments[v->n_segments - 1]->duration :
+                                  v->target_duration;
+        reload_interval *= 1000000;
+
         if (!v->finished &&
-            av_gettime() - v->last_load_time >= v->target_duration*1000000 &&
+            av_gettime() - v->last_load_time >= reload_interval &&
             (ret = parse_playlist(c, v->url, v, NULL)) < 0)
                 return ret;
         if (v->cur_seq_no < v->start_seq_no) {
@@ -392,8 +397,7 @@ reload:
         if (v->cur_seq_no >= v->start_seq_no + v->n_segments) {
             if (v->finished)
                 return AVERROR_EOF;
-            while (av_gettime() - v->last_load_time <
-                   v->target_duration*1000000) {
+            while (av_gettime() - v->last_load_time < reload_interval) {
                 if (ff_check_interrupt(c->interrupt_callback))
                     return AVERROR_EXIT;
                 usleep(100*1000);
@@ -377,10 +377,15 @@ static int read_data(void *opaque, uint8_t *buf, int buf_size)
     if (!v->input) {
 reload:
-        /* If this is a live stream and target_duration has elapsed since
+        /* If this is a live stream and the reload interval has elapsed since
          * the last playlist reload, reload the variant playlists now. */